"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseKeys = void 0;
const string_decoder_1 = require("string_decoder");
function parseKeys(data) {
    const keys = [];
    let context = null;
    const createEscapeSequenceContext = () => {
        let state = null;
        let buffer = [];
        let key = null;
        const setKey = () => {
            key = Buffer.from(buffer);
            buffer = [];
        };
        return {
            end: () => {
                return buffer.length === 0 ? null : Buffer.from(buffer);
            },
            push: (char) => {
                buffer.push(char);
                switch (state) {
                    case null: {
                        state = 'escape-symbol';
                        break;
                    }
                    case 'escape-symbol': {
                        if (char === 0x5b || char == 0x4f || char === 0x4e) {
                            state = 'parameters';
                        }
                        else {
                            setKey();
                        }
                        break;
                    }
                    case 'parameters': {
                        if ((char >= 0x41 && char <= 0x5a)
                            || (char >= 0x61 && char <= 0x7a)
                            || char === 0x7e) {
                            setKey();
                        }
                        break;
                    }
                    default: {
                        throw new Error('Unknown state');
                    }
                }
            },
            getKey: () => key
        };
    };
    const createStringContext = () => {
        const decoder = new string_decoder_1.StringDecoder('utf-8');
        let key = null;
        return {
            end: () => {
                const rest = decoder.end();
                return rest === '' ? null : rest;
            },
            push: (char) => {
                const result = decoder.write(Buffer.from([char]));
                if (result !== '') {
                    key = Buffer.from(result, 'utf-8');
                }
            },
            getKey: () => key
        };
    };
    for (let char of data) {
        if (context === null) {
            if (char === 0x1b) {
                context = createEscapeSequenceContext();
            }
            else {
                context = createStringContext();
            }
        }
        context.push(char);
        const key = context.getKey();
        if (key !== null) {
            keys.push(key);
            context = null;
        }
    }
    const unparsedChars = context === null ? null : context.end();
    if (unparsedChars !== null) {
        keys.push(unparsedChars);
    }
    return keys;
}
exports.parseKeys = parseKeys;
//# sourceMappingURL=parseKeys.js.map